Package: nlmrt
Type: Package
Title: Functions for Nonlinear Least Squares Solutions
Version: 2016.3.2
Date: 2016-3-2
Authors@R: c( person(given = c("John", "C."), family = "Nash", role =
        c("aut", "cre"), email = "nashjc@uottawa.ca") )
Author: John C. Nash [aut, cre]
Maintainer: John C. Nash <nashjc@uottawa.ca>
Description: Replacement for nls() tools for working with nonlinear least squares problems.
      The calling structure is similar to, but much simpler than, that of the nls()
      function. Moreover, where nls() specifically does NOT deal with small or zero
      residual problems, nlmrt is quite happy to solve them. It also attempts to be
      more robust in finding solutions, thereby avoiding 'singular gradient' messages
      that arise in the Gauss-Newton method within nls(). The Marquardt-Nash approach
      in nlmrt generally works more reliably to get a solution, though this may be 
      one of a set of possibilities, and may also be statistically unsatisfactory.
      Added print and summary as of August 28, 2012.
License: GPL-2
Depends: R (>= 2.15.0)
Suggests: minpack.lm, optimx, Rvmmin, Rcgmin, numDeriv
NeedsCompilation: no
Packaged: 2016-03-04 16:30:35 UTC; john
Repository: CRAN
Date/Publication: 2016-03-04 23:57:39
Built: R 4.6.0; ; 2026-04-25 20:46:34 UTC; unix
